# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ems_grpc.proto
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import sys


_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="ems_grpc.proto",
    package="IOSXRExtensibleManagabilityService",
    syntax="proto3",
    serialized_pb=_b(
        '\n\x0e\x65ms_grpc.proto\x12"IOSXRExtensibleManagabilityService"4\n\rConfigGetArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x14\n\x0cyangpathjson\x18\x02 \x01(\t"D\n\x0e\x43onfigGetReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x10\n\x08yangjson\x18\x02 \x01(\t\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t"2\n\x0bGetOperArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x14\n\x0cyangpathjson\x18\x02 \x01(\t"B\n\x0cGetOperReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x10\n\x08yangjson\x18\x02 \x01(\t\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t"-\n\nConfigArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x10\n\x08yangjson\x18\x02 \x01(\t"/\n\x0b\x43onfigReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0e\n\x06\x65rrors\x18\x02 \x01(\t"+\n\rCliConfigArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x0b\n\x03\x63li\x18\x02 \x01(\t"2\n\x0e\x43liConfigReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0e\n\x06\x65rrors\x18\x02 \x01(\t"A\n\x11\x43ommitReplaceArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x0b\n\x03\x63li\x18\x02 \x01(\t\x12\x10\n\x08yangjson\x18\x03 \x01(\t"6\n\x12\x43ommitReplaceReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0e\n\x06\x65rrors\x18\x02 \x01(\t"+\n\tCommitMsg\x12\r\n\x05label\x18\x01 \x01(\t\x12\x0f\n\x07\x63omment\x18\x02 \x01(\t"W\n\nCommitArgs\x12:\n\x03msg\x18\x01 \x01(\x0b\x32-.IOSXRExtensibleManagabilityService.CommitMsg\x12\r\n\x05ReqId\x18\x02 \x01(\x03"q\n\x0b\x43ommitReply\x12@\n\x06result\x18\x01 \x01(\x0e\x32\x30.IOSXRExtensibleManagabilityService.CommitResult\x12\x10\n\x08ResReqId\x18\x02 \x01(\x03\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t"#\n\x12\x44iscardChangesArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03"7\n\x13\x44iscardChangesReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0e\n\x06\x65rrors\x18\x02 \x01(\t")\n\x0bShowCmdArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x0b\n\x03\x63li\x18\x02 \x01(\t"D\n\x10ShowCmdTextReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0e\n\x06output\x18\x02 \x01(\t\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t"H\n\x10ShowCmdJSONReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x12\n\njsonoutput\x18\x02 \x01(\t\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t"A\n\x0e\x43reateSubsArgs\x12\r\n\x05ReqId\x18\x01 \x01(\x03\x12\x0e\n\x06\x65ncode\x18\x02 \x01(\x03\x12\x10\n\x08subidstr\x18\x03 \x01(\t"A\n\x0f\x43reateSubsReply\x12\x10\n\x08ResReqId\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0e\n\x06\x65rrors\x18\x03 \x01(\t*3\n\x0c\x43ommitResult\x12\n\n\x06\x43HANGE\x10\x00\x12\r\n\tNO_CHANGE\x10\x01\x12\x08\n\x04\x46\x41IL\x10\x02\x32\xc6\t\n\x0egRPCConfigOper\x12v\n\tGetConfig\x12\x31.IOSXRExtensibleManagabilityService.ConfigGetArgs\x1a\x32.IOSXRExtensibleManagabilityService.ConfigGetReply"\x00\x30\x01\x12p\n\x0bMergeConfig\x12..IOSXRExtensibleManagabilityService.ConfigArgs\x1a/.IOSXRExtensibleManagabilityService.ConfigReply"\x00\x12q\n\x0c\x44\x65leteConfig\x12..IOSXRExtensibleManagabilityService.ConfigArgs\x1a/.IOSXRExtensibleManagabilityService.ConfigReply"\x00\x12r\n\rReplaceConfig\x12..IOSXRExtensibleManagabilityService.ConfigArgs\x1a/.IOSXRExtensibleManagabilityService.ConfigReply"\x00\x12t\n\tCliConfig\x12\x31.IOSXRExtensibleManagabilityService.CliConfigArgs\x1a\x32.IOSXRExtensibleManagabilityService.CliConfigReply"\x00\x12\x80\x01\n\rCommitReplace\x12\x35.IOSXRExtensibleManagabilityService.CommitReplaceArgs\x1a\x36.IOSXRExtensibleManagabilityService.CommitReplaceReply"\x00\x12q\n\x0c\x43ommitConfig\x12..IOSXRExtensibleManagabilityService.CommitArgs\x1a/.IOSXRExtensibleManagabilityService.CommitReply"\x00\x12\x89\x01\n\x14\x43onfigDiscardChanges\x12\x36.IOSXRExtensibleManagabilityService.DiscardChangesArgs\x1a\x37.IOSXRExtensibleManagabilityService.DiscardChangesReply"\x00\x12p\n\x07GetOper\x12/.IOSXRExtensibleManagabilityService.GetOperArgs\x1a\x30.IOSXRExtensibleManagabilityService.GetOperReply"\x00\x30\x01\x12y\n\nCreateSubs\x12\x32.IOSXRExtensibleManagabilityService.CreateSubsArgs\x1a\x33.IOSXRExtensibleManagabilityService.CreateSubsReply"\x00\x30\x01\x32\x8a\x02\n\x08gRPCExec\x12~\n\x11ShowCmdTextOutput\x12/.IOSXRExtensibleManagabilityService.ShowCmdArgs\x1a\x34.IOSXRExtensibleManagabilityService.ShowCmdTextReply"\x00\x30\x01\x12~\n\x11ShowCmdJSONOutput\x12/.IOSXRExtensibleManagabilityService.ShowCmdArgs\x1a\x34.IOSXRExtensibleManagabilityService.ShowCmdJSONReply"\x00\x30\x01\x62\x06proto3',  # noqa: E501
    ),
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_COMMITRESULT = _descriptor.EnumDescriptor(
    name="CommitResult",
    full_name="IOSXRExtensibleManagabilityService.CommitResult",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="CHANGE",
            index=0,
            number=0,
            options=None,
            type=None,
        ),
        _descriptor.EnumValueDescriptor(
            name="NO_CHANGE",
            index=1,
            number=1,
            options=None,
            type=None,
        ),
        _descriptor.EnumValueDescriptor(
            name="FAIL",
            index=2,
            number=2,
            options=None,
            type=None,
        ),
    ],
    containing_type=None,
    options=None,
    serialized_start=1278,
    serialized_end=1329,
)
_sym_db.RegisterEnumDescriptor(_COMMITRESULT)

CommitResult = enum_type_wrapper.EnumTypeWrapper(_COMMITRESULT)
CHANGE = 0
NO_CHANGE = 1
FAIL = 2


_CONFIGGETARGS = _descriptor.Descriptor(
    name="ConfigGetArgs",
    full_name="IOSXRExtensibleManagabilityService.ConfigGetArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.ConfigGetArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangpathjson",
            full_name="IOSXRExtensibleManagabilityService.ConfigGetArgs.yangpathjson",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=54,
    serialized_end=106,
)


_CONFIGGETREPLY = _descriptor.Descriptor(
    name="ConfigGetReply",
    full_name="IOSXRExtensibleManagabilityService.ConfigGetReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.ConfigGetReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangjson",
            full_name="IOSXRExtensibleManagabilityService.ConfigGetReply.yangjson",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.ConfigGetReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=108,
    serialized_end=176,
)


_GETOPERARGS = _descriptor.Descriptor(
    name="GetOperArgs",
    full_name="IOSXRExtensibleManagabilityService.GetOperArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.GetOperArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangpathjson",
            full_name="IOSXRExtensibleManagabilityService.GetOperArgs.yangpathjson",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=178,
    serialized_end=228,
)


_GETOPERREPLY = _descriptor.Descriptor(
    name="GetOperReply",
    full_name="IOSXRExtensibleManagabilityService.GetOperReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.GetOperReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangjson",
            full_name="IOSXRExtensibleManagabilityService.GetOperReply.yangjson",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.GetOperReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=230,
    serialized_end=296,
)


_CONFIGARGS = _descriptor.Descriptor(
    name="ConfigArgs",
    full_name="IOSXRExtensibleManagabilityService.ConfigArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.ConfigArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangjson",
            full_name="IOSXRExtensibleManagabilityService.ConfigArgs.yangjson",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=298,
    serialized_end=343,
)


_CONFIGREPLY = _descriptor.Descriptor(
    name="ConfigReply",
    full_name="IOSXRExtensibleManagabilityService.ConfigReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.ConfigReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.ConfigReply.errors",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=345,
    serialized_end=392,
)


_CLICONFIGARGS = _descriptor.Descriptor(
    name="CliConfigArgs",
    full_name="IOSXRExtensibleManagabilityService.CliConfigArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.CliConfigArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="cli",
            full_name="IOSXRExtensibleManagabilityService.CliConfigArgs.cli",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=394,
    serialized_end=437,
)


_CLICONFIGREPLY = _descriptor.Descriptor(
    name="CliConfigReply",
    full_name="IOSXRExtensibleManagabilityService.CliConfigReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.CliConfigReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.CliConfigReply.errors",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=439,
    serialized_end=489,
)


_COMMITREPLACEARGS = _descriptor.Descriptor(
    name="CommitReplaceArgs",
    full_name="IOSXRExtensibleManagabilityService.CommitReplaceArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.CommitReplaceArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="cli",
            full_name="IOSXRExtensibleManagabilityService.CommitReplaceArgs.cli",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="yangjson",
            full_name="IOSXRExtensibleManagabilityService.CommitReplaceArgs.yangjson",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=491,
    serialized_end=556,
)


_COMMITREPLACEREPLY = _descriptor.Descriptor(
    name="CommitReplaceReply",
    full_name="IOSXRExtensibleManagabilityService.CommitReplaceReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.CommitReplaceReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.CommitReplaceReply.errors",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=558,
    serialized_end=612,
)


_COMMITMSG = _descriptor.Descriptor(
    name="CommitMsg",
    full_name="IOSXRExtensibleManagabilityService.CommitMsg",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="label",
            full_name="IOSXRExtensibleManagabilityService.CommitMsg.label",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="comment",
            full_name="IOSXRExtensibleManagabilityService.CommitMsg.comment",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=614,
    serialized_end=657,
)


_COMMITARGS = _descriptor.Descriptor(
    name="CommitArgs",
    full_name="IOSXRExtensibleManagabilityService.CommitArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="msg",
            full_name="IOSXRExtensibleManagabilityService.CommitArgs.msg",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.CommitArgs.ReqId",
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=659,
    serialized_end=746,
)


_COMMITREPLY = _descriptor.Descriptor(
    name="CommitReply",
    full_name="IOSXRExtensibleManagabilityService.CommitReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="result",
            full_name="IOSXRExtensibleManagabilityService.CommitReply.result",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.CommitReply.ResReqId",
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.CommitReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=748,
    serialized_end=861,
)


_DISCARDCHANGESARGS = _descriptor.Descriptor(
    name="DiscardChangesArgs",
    full_name="IOSXRExtensibleManagabilityService.DiscardChangesArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.DiscardChangesArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=863,
    serialized_end=898,
)


_DISCARDCHANGESREPLY = _descriptor.Descriptor(
    name="DiscardChangesReply",
    full_name="IOSXRExtensibleManagabilityService.DiscardChangesReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.DiscardChangesReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.DiscardChangesReply.errors",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=900,
    serialized_end=955,
)


_SHOWCMDARGS = _descriptor.Descriptor(
    name="ShowCmdArgs",
    full_name="IOSXRExtensibleManagabilityService.ShowCmdArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="cli",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdArgs.cli",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=957,
    serialized_end=998,
)


_SHOWCMDTEXTREPLY = _descriptor.Descriptor(
    name="ShowCmdTextReply",
    full_name="IOSXRExtensibleManagabilityService.ShowCmdTextReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdTextReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="output",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdTextReply.output",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdTextReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1000,
    serialized_end=1068,
)


_SHOWCMDJSONREPLY = _descriptor.Descriptor(
    name="ShowCmdJSONReply",
    full_name="IOSXRExtensibleManagabilityService.ShowCmdJSONReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdJSONReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="jsonoutput",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdJSONReply.jsonoutput",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.ShowCmdJSONReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1070,
    serialized_end=1142,
)


_CREATESUBSARGS = _descriptor.Descriptor(
    name="CreateSubsArgs",
    full_name="IOSXRExtensibleManagabilityService.CreateSubsArgs",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ReqId",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsArgs.ReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="encode",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsArgs.encode",
            index=1,
            number=2,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="subidstr",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsArgs.subidstr",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1144,
    serialized_end=1209,
)


_CREATESUBSREPLY = _descriptor.Descriptor(
    name="CreateSubsReply",
    full_name="IOSXRExtensibleManagabilityService.CreateSubsReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="ResReqId",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsReply.ResReqId",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsReply.data",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b(""),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
        _descriptor.FieldDescriptor(
            name="errors",
            full_name="IOSXRExtensibleManagabilityService.CreateSubsReply.errors",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            options=None,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1211,
    serialized_end=1276,
)

_COMMITARGS.fields_by_name["msg"].message_type = _COMMITMSG
_COMMITREPLY.fields_by_name["result"].enum_type = _COMMITRESULT
DESCRIPTOR.message_types_by_name["ConfigGetArgs"] = _CONFIGGETARGS
DESCRIPTOR.message_types_by_name["ConfigGetReply"] = _CONFIGGETREPLY
DESCRIPTOR.message_types_by_name["GetOperArgs"] = _GETOPERARGS
DESCRIPTOR.message_types_by_name["GetOperReply"] = _GETOPERREPLY
DESCRIPTOR.message_types_by_name["ConfigArgs"] = _CONFIGARGS
DESCRIPTOR.message_types_by_name["ConfigReply"] = _CONFIGREPLY
DESCRIPTOR.message_types_by_name["CliConfigArgs"] = _CLICONFIGARGS
DESCRIPTOR.message_types_by_name["CliConfigReply"] = _CLICONFIGREPLY
DESCRIPTOR.message_types_by_name["CommitReplaceArgs"] = _COMMITREPLACEARGS
DESCRIPTOR.message_types_by_name["CommitReplaceReply"] = _COMMITREPLACEREPLY
DESCRIPTOR.message_types_by_name["CommitMsg"] = _COMMITMSG
DESCRIPTOR.message_types_by_name["CommitArgs"] = _COMMITARGS
DESCRIPTOR.message_types_by_name["CommitReply"] = _COMMITREPLY
DESCRIPTOR.message_types_by_name["DiscardChangesArgs"] = _DISCARDCHANGESARGS
DESCRIPTOR.message_types_by_name["DiscardChangesReply"] = _DISCARDCHANGESREPLY
DESCRIPTOR.message_types_by_name["ShowCmdArgs"] = _SHOWCMDARGS
DESCRIPTOR.message_types_by_name["ShowCmdTextReply"] = _SHOWCMDTEXTREPLY
DESCRIPTOR.message_types_by_name["ShowCmdJSONReply"] = _SHOWCMDJSONREPLY
DESCRIPTOR.message_types_by_name["CreateSubsArgs"] = _CREATESUBSARGS
DESCRIPTOR.message_types_by_name["CreateSubsReply"] = _CREATESUBSREPLY
DESCRIPTOR.enum_types_by_name["CommitResult"] = _COMMITRESULT

ConfigGetArgs = _reflection.GeneratedProtocolMessageType(
    "ConfigGetArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONFIGGETARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ConfigGetArgs)
    ),
)
_sym_db.RegisterMessage(ConfigGetArgs)

ConfigGetReply = _reflection.GeneratedProtocolMessageType(
    "ConfigGetReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONFIGGETREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ConfigGetReply)
    ),
)
_sym_db.RegisterMessage(ConfigGetReply)

GetOperArgs = _reflection.GeneratedProtocolMessageType(
    "GetOperArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_GETOPERARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.GetOperArgs)
    ),
)
_sym_db.RegisterMessage(GetOperArgs)

GetOperReply = _reflection.GeneratedProtocolMessageType(
    "GetOperReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_GETOPERREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.GetOperReply)
    ),
)
_sym_db.RegisterMessage(GetOperReply)

ConfigArgs = _reflection.GeneratedProtocolMessageType(
    "ConfigArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONFIGARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ConfigArgs)
    ),
)
_sym_db.RegisterMessage(ConfigArgs)

ConfigReply = _reflection.GeneratedProtocolMessageType(
    "ConfigReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONFIGREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ConfigReply)
    ),
)
_sym_db.RegisterMessage(ConfigReply)

CliConfigArgs = _reflection.GeneratedProtocolMessageType(
    "CliConfigArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CLICONFIGARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CliConfigArgs)
    ),
)
_sym_db.RegisterMessage(CliConfigArgs)

CliConfigReply = _reflection.GeneratedProtocolMessageType(
    "CliConfigReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CLICONFIGREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CliConfigReply)
    ),
)
_sym_db.RegisterMessage(CliConfigReply)

CommitReplaceArgs = _reflection.GeneratedProtocolMessageType(
    "CommitReplaceArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COMMITREPLACEARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CommitReplaceArgs)
    ),
)
_sym_db.RegisterMessage(CommitReplaceArgs)

CommitReplaceReply = _reflection.GeneratedProtocolMessageType(
    "CommitReplaceReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COMMITREPLACEREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CommitReplaceReply)
    ),
)
_sym_db.RegisterMessage(CommitReplaceReply)

CommitMsg = _reflection.GeneratedProtocolMessageType(
    "CommitMsg",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COMMITMSG,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CommitMsg)
    ),
)
_sym_db.RegisterMessage(CommitMsg)

CommitArgs = _reflection.GeneratedProtocolMessageType(
    "CommitArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COMMITARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CommitArgs)
    ),
)
_sym_db.RegisterMessage(CommitArgs)

CommitReply = _reflection.GeneratedProtocolMessageType(
    "CommitReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COMMITREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CommitReply)
    ),
)
_sym_db.RegisterMessage(CommitReply)

DiscardChangesArgs = _reflection.GeneratedProtocolMessageType(
    "DiscardChangesArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DISCARDCHANGESARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.DiscardChangesArgs)
    ),
)
_sym_db.RegisterMessage(DiscardChangesArgs)

DiscardChangesReply = _reflection.GeneratedProtocolMessageType(
    "DiscardChangesReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DISCARDCHANGESREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.DiscardChangesReply)
    ),
)
_sym_db.RegisterMessage(DiscardChangesReply)

ShowCmdArgs = _reflection.GeneratedProtocolMessageType(
    "ShowCmdArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SHOWCMDARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ShowCmdArgs)
    ),
)
_sym_db.RegisterMessage(ShowCmdArgs)

ShowCmdTextReply = _reflection.GeneratedProtocolMessageType(
    "ShowCmdTextReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SHOWCMDTEXTREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ShowCmdTextReply)
    ),
)
_sym_db.RegisterMessage(ShowCmdTextReply)

ShowCmdJSONReply = _reflection.GeneratedProtocolMessageType(
    "ShowCmdJSONReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SHOWCMDJSONREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.ShowCmdJSONReply)
    ),
)
_sym_db.RegisterMessage(ShowCmdJSONReply)

CreateSubsArgs = _reflection.GeneratedProtocolMessageType(
    "CreateSubsArgs",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CREATESUBSARGS,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CreateSubsArgs)
    ),
)
_sym_db.RegisterMessage(CreateSubsArgs)

CreateSubsReply = _reflection.GeneratedProtocolMessageType(
    "CreateSubsReply",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CREATESUBSREPLY,
        __module__="ems_grpc_pb2",
        # @@protoc_insertion_point(class_scope:IOSXRExtensibleManagabilityService.CreateSubsReply)
    ),
)
_sym_db.RegisterMessage(CreateSubsReply)


import grpc

from grpc.beta import implementations as beta_implementations
from grpc.beta import interfaces as beta_interfaces
from grpc.framework.common import cardinality
from grpc.framework.interfaces.face import utilities as face_utilities


class gRPCConfigOperStub(object):
    def __init__(self, channel):
        """Constructor.

        Args:
          channel: A grpc.Channel.
        """
        self.GetConfig = channel.unary_stream(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/GetConfig",
            request_serializer=ConfigGetArgs.SerializeToString,
            response_deserializer=ConfigGetReply.FromString,
        )
        self.MergeConfig = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/MergeConfig",
            request_serializer=ConfigArgs.SerializeToString,
            response_deserializer=ConfigReply.FromString,
        )
        self.DeleteConfig = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/DeleteConfig",
            request_serializer=ConfigArgs.SerializeToString,
            response_deserializer=ConfigReply.FromString,
        )
        self.ReplaceConfig = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/ReplaceConfig",
            request_serializer=ConfigArgs.SerializeToString,
            response_deserializer=ConfigReply.FromString,
        )
        self.CliConfig = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/CliConfig",
            request_serializer=CliConfigArgs.SerializeToString,
            response_deserializer=CliConfigReply.FromString,
        )
        self.CommitReplace = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/CommitReplace",
            request_serializer=CommitReplaceArgs.SerializeToString,
            response_deserializer=CommitReplaceReply.FromString,
        )
        self.CommitConfig = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/CommitConfig",
            request_serializer=CommitArgs.SerializeToString,
            response_deserializer=CommitReply.FromString,
        )
        self.ConfigDiscardChanges = channel.unary_unary(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/ConfigDiscardChanges",
            request_serializer=DiscardChangesArgs.SerializeToString,
            response_deserializer=DiscardChangesReply.FromString,
        )
        self.GetOper = channel.unary_stream(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/GetOper",
            request_serializer=GetOperArgs.SerializeToString,
            response_deserializer=GetOperReply.FromString,
        )
        self.CreateSubs = channel.unary_stream(
            "/IOSXRExtensibleManagabilityService.gRPCConfigOper/CreateSubs",
            request_serializer=CreateSubsArgs.SerializeToString,
            response_deserializer=CreateSubsReply.FromString,
        )


class gRPCConfigOperServicer(object):
    def GetConfig(self, request, context):
        """Configuration related commands"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def MergeConfig(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteConfig(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ReplaceConfig(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CliConfig(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CommitReplace(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CommitConfig(self, request, context):
        """Do we need implicit or explicit commit"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ConfigDiscardChanges(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetOper(self, request, context):
        """Get only returns oper data"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateSubs(self, request, context):
        """Get Telemetry Data"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_gRPCConfigOperServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetConfig": grpc.unary_stream_rpc_method_handler(
            servicer.GetConfig,
            request_deserializer=ConfigGetArgs.FromString,
            response_serializer=ConfigGetReply.SerializeToString,
        ),
        "MergeConfig": grpc.unary_unary_rpc_method_handler(
            servicer.MergeConfig,
            request_deserializer=ConfigArgs.FromString,
            response_serializer=ConfigReply.SerializeToString,
        ),
        "DeleteConfig": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteConfig,
            request_deserializer=ConfigArgs.FromString,
            response_serializer=ConfigReply.SerializeToString,
        ),
        "ReplaceConfig": grpc.unary_unary_rpc_method_handler(
            servicer.ReplaceConfig,
            request_deserializer=ConfigArgs.FromString,
            response_serializer=ConfigReply.SerializeToString,
        ),
        "CliConfig": grpc.unary_unary_rpc_method_handler(
            servicer.CliConfig,
            request_deserializer=CliConfigArgs.FromString,
            response_serializer=CliConfigReply.SerializeToString,
        ),
        "CommitReplace": grpc.unary_unary_rpc_method_handler(
            servicer.CommitReplace,
            request_deserializer=CommitReplaceArgs.FromString,
            response_serializer=CommitReplaceReply.SerializeToString,
        ),
        "CommitConfig": grpc.unary_unary_rpc_method_handler(
            servicer.CommitConfig,
            request_deserializer=CommitArgs.FromString,
            response_serializer=CommitReply.SerializeToString,
        ),
        "ConfigDiscardChanges": grpc.unary_unary_rpc_method_handler(
            servicer.ConfigDiscardChanges,
            request_deserializer=DiscardChangesArgs.FromString,
            response_serializer=DiscardChangesReply.SerializeToString,
        ),
        "GetOper": grpc.unary_stream_rpc_method_handler(
            servicer.GetOper,
            request_deserializer=GetOperArgs.FromString,
            response_serializer=GetOperReply.SerializeToString,
        ),
        "CreateSubs": grpc.unary_stream_rpc_method_handler(
            servicer.CreateSubs,
            request_deserializer=CreateSubsArgs.FromString,
            response_serializer=CreateSubsReply.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "IOSXRExtensibleManagabilityService.gRPCConfigOper",
        rpc_method_handlers,
    )
    server.add_generic_rpc_handlers((generic_handler,))


class BetagRPCConfigOperServicer(object):
    def GetConfig(self, request, context):
        """Configuration related commands"""
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def MergeConfig(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def DeleteConfig(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def ReplaceConfig(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def CliConfig(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def CommitReplace(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def CommitConfig(self, request, context):
        """Do we need implicit or explicit commit"""
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def ConfigDiscardChanges(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def GetOper(self, request, context):
        """Get only returns oper data"""
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def CreateSubs(self, request, context):
        """Get Telemetry Data"""
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)


class BetagRPCConfigOperStub(object):
    def GetConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        """Configuration related commands"""
        raise NotImplementedError()

    def MergeConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    MergeConfig.future = None

    def DeleteConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    DeleteConfig.future = None

    def ReplaceConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    ReplaceConfig.future = None

    def CliConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    CliConfig.future = None

    def CommitReplace(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    CommitReplace.future = None

    def CommitConfig(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        """Do we need implicit or explicit commit"""
        raise NotImplementedError()

    CommitConfig.future = None

    def ConfigDiscardChanges(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()

    ConfigDiscardChanges.future = None

    def GetOper(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        """Get only returns oper data"""
        raise NotImplementedError()

    def CreateSubs(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        """Get Telemetry Data"""
        raise NotImplementedError()


def beta_create_gRPCConfigOper_server(
    servicer,
    pool=None,
    pool_size=None,
    default_timeout=None,
    maximum_timeout=None,
):
    request_deserializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CliConfig",
        ): CliConfigArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitConfig",
        ): CommitArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitReplace",
        ): CommitReplaceArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ConfigDiscardChanges",
        ): DiscardChangesArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CreateSubs",
        ): CreateSubsArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "DeleteConfig",
        ): ConfigArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetConfig",
        ): ConfigGetArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetOper",
        ): GetOperArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "MergeConfig",
        ): ConfigArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ReplaceConfig",
        ): ConfigArgs.FromString,
    }
    response_serializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CliConfig",
        ): CliConfigReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitConfig",
        ): CommitReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitReplace",
        ): CommitReplaceReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ConfigDiscardChanges",
        ): DiscardChangesReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CreateSubs",
        ): CreateSubsReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "DeleteConfig",
        ): ConfigReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetConfig",
        ): ConfigGetReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetOper",
        ): GetOperReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "MergeConfig",
        ): ConfigReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ReplaceConfig",
        ): ConfigReply.SerializeToString,
    }
    method_implementations = {
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CliConfig",
        ): face_utilities.unary_unary_inline(servicer.CliConfig),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitConfig",
        ): face_utilities.unary_unary_inline(servicer.CommitConfig),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitReplace",
        ): face_utilities.unary_unary_inline(servicer.CommitReplace),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ConfigDiscardChanges",
        ): face_utilities.unary_unary_inline(servicer.ConfigDiscardChanges),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CreateSubs",
        ): face_utilities.unary_stream_inline(servicer.CreateSubs),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "DeleteConfig",
        ): face_utilities.unary_unary_inline(servicer.DeleteConfig),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetConfig",
        ): face_utilities.unary_stream_inline(servicer.GetConfig),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetOper",
        ): face_utilities.unary_stream_inline(servicer.GetOper),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "MergeConfig",
        ): face_utilities.unary_unary_inline(servicer.MergeConfig),
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ReplaceConfig",
        ): face_utilities.unary_unary_inline(servicer.ReplaceConfig),
    }
    server_options = beta_implementations.server_options(
        request_deserializers=request_deserializers,
        response_serializers=response_serializers,
        thread_pool=pool,
        thread_pool_size=pool_size,
        default_timeout=default_timeout,
        maximum_timeout=maximum_timeout,
    )
    return beta_implementations.server(
        method_implementations,
        options=server_options,
    )


def beta_create_gRPCConfigOper_stub(
    channel,
    host=None,
    metadata_transformer=None,
    pool=None,
    pool_size=None,
):
    request_serializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CliConfig",
        ): CliConfigArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitConfig",
        ): CommitArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitReplace",
        ): CommitReplaceArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ConfigDiscardChanges",
        ): DiscardChangesArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CreateSubs",
        ): CreateSubsArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "DeleteConfig",
        ): ConfigArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetConfig",
        ): ConfigGetArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetOper",
        ): GetOperArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "MergeConfig",
        ): ConfigArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ReplaceConfig",
        ): ConfigArgs.SerializeToString,
    }
    response_deserializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CliConfig",
        ): CliConfigReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitConfig",
        ): CommitReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CommitReplace",
        ): CommitReplaceReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ConfigDiscardChanges",
        ): DiscardChangesReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "CreateSubs",
        ): CreateSubsReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "DeleteConfig",
        ): ConfigReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetConfig",
        ): ConfigGetReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "GetOper",
        ): GetOperReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "MergeConfig",
        ): ConfigReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCConfigOper",
            "ReplaceConfig",
        ): ConfigReply.FromString,
    }
    cardinalities = {
        "CliConfig": cardinality.Cardinality.UNARY_UNARY,
        "CommitConfig": cardinality.Cardinality.UNARY_UNARY,
        "CommitReplace": cardinality.Cardinality.UNARY_UNARY,
        "ConfigDiscardChanges": cardinality.Cardinality.UNARY_UNARY,
        "CreateSubs": cardinality.Cardinality.UNARY_STREAM,
        "DeleteConfig": cardinality.Cardinality.UNARY_UNARY,
        "GetConfig": cardinality.Cardinality.UNARY_STREAM,
        "GetOper": cardinality.Cardinality.UNARY_STREAM,
        "MergeConfig": cardinality.Cardinality.UNARY_UNARY,
        "ReplaceConfig": cardinality.Cardinality.UNARY_UNARY,
    }
    stub_options = beta_implementations.stub_options(
        host=host,
        metadata_transformer=metadata_transformer,
        request_serializers=request_serializers,
        response_deserializers=response_deserializers,
        thread_pool=pool,
        thread_pool_size=pool_size,
    )
    return beta_implementations.dynamic_stub(
        channel,
        "IOSXRExtensibleManagabilityService.gRPCConfigOper",
        cardinalities,
        options=stub_options,
    )


class gRPCExecStub(object):
    """
    Should we seperate Exec from Config/Oper?


    """

    def __init__(self, channel):
        """Constructor.

        Args:
          channel: A grpc.Channel.
        """
        self.ShowCmdTextOutput = channel.unary_stream(
            "/IOSXRExtensibleManagabilityService.gRPCExec/ShowCmdTextOutput",
            request_serializer=ShowCmdArgs.SerializeToString,
            response_deserializer=ShowCmdTextReply.FromString,
        )
        self.ShowCmdJSONOutput = channel.unary_stream(
            "/IOSXRExtensibleManagabilityService.gRPCExec/ShowCmdJSONOutput",
            request_serializer=ShowCmdArgs.SerializeToString,
            response_deserializer=ShowCmdJSONReply.FromString,
        )


class gRPCExecServicer(object):
    """
    Should we seperate Exec from Config/Oper?


    """

    def ShowCmdTextOutput(self, request, context):
        """Exec commands"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ShowCmdJSONOutput(self, request, context):
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_gRPCExecServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ShowCmdTextOutput": grpc.unary_stream_rpc_method_handler(
            servicer.ShowCmdTextOutput,
            request_deserializer=ShowCmdArgs.FromString,
            response_serializer=ShowCmdTextReply.SerializeToString,
        ),
        "ShowCmdJSONOutput": grpc.unary_stream_rpc_method_handler(
            servicer.ShowCmdJSONOutput,
            request_deserializer=ShowCmdArgs.FromString,
            response_serializer=ShowCmdJSONReply.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "IOSXRExtensibleManagabilityService.gRPCExec",
        rpc_method_handlers,
    )
    server.add_generic_rpc_handlers((generic_handler,))


class BetagRPCExecServicer(object):
    """
    Should we seperate Exec from Config/Oper?


    """

    def ShowCmdTextOutput(self, request, context):
        """Exec commands"""
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def ShowCmdJSONOutput(self, request, context):
        context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)


class BetagRPCExecStub(object):
    """
    Should we seperate Exec from Config/Oper?


    """

    def ShowCmdTextOutput(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        """Exec commands"""
        raise NotImplementedError()

    def ShowCmdJSONOutput(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
        raise NotImplementedError()


def beta_create_gRPCExec_server(
    servicer,
    pool=None,
    pool_size=None,
    default_timeout=None,
    maximum_timeout=None,
):
    request_deserializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdJSONOutput",
        ): ShowCmdArgs.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdTextOutput",
        ): ShowCmdArgs.FromString,
    }
    response_serializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdJSONOutput",
        ): ShowCmdJSONReply.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdTextOutput",
        ): ShowCmdTextReply.SerializeToString,
    }
    method_implementations = {
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdJSONOutput",
        ): face_utilities.unary_stream_inline(servicer.ShowCmdJSONOutput),
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdTextOutput",
        ): face_utilities.unary_stream_inline(servicer.ShowCmdTextOutput),
    }
    server_options = beta_implementations.server_options(
        request_deserializers=request_deserializers,
        response_serializers=response_serializers,
        thread_pool=pool,
        thread_pool_size=pool_size,
        default_timeout=default_timeout,
        maximum_timeout=maximum_timeout,
    )
    return beta_implementations.server(
        method_implementations,
        options=server_options,
    )


def beta_create_gRPCExec_stub(
    channel,
    host=None,
    metadata_transformer=None,
    pool=None,
    pool_size=None,
):
    request_serializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdJSONOutput",
        ): ShowCmdArgs.SerializeToString,
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdTextOutput",
        ): ShowCmdArgs.SerializeToString,
    }
    response_deserializers = {
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdJSONOutput",
        ): ShowCmdJSONReply.FromString,
        (
            "IOSXRExtensibleManagabilityService.gRPCExec",
            "ShowCmdTextOutput",
        ): ShowCmdTextReply.FromString,
    }
    cardinalities = {
        "ShowCmdJSONOutput": cardinality.Cardinality.UNARY_STREAM,
        "ShowCmdTextOutput": cardinality.Cardinality.UNARY_STREAM,
    }
    stub_options = beta_implementations.stub_options(
        host=host,
        metadata_transformer=metadata_transformer,
        request_serializers=request_serializers,
        response_deserializers=response_deserializers,
        thread_pool=pool,
        thread_pool_size=pool_size,
    )
    return beta_implementations.dynamic_stub(
        channel,
        "IOSXRExtensibleManagabilityService.gRPCExec",
        cardinalities,
        options=stub_options,
    )


# @@protoc_insertion_point(module_scope)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: sns_topic_info
short_description: sns_topic_info module
version_added: 3.2.0
description:
- The M(community.aws.sns_topic_info) module allows to get all AWS SNS topics or properties of a specific AWS SNS topic.
author:
- "Alina Buzachis (@alinabuzachis)"
options:
    topic_arn:
        description: The ARN of the AWS SNS topic for which you wish to find subscriptions or list attributes.
        required: false
        type: str
extends_documentation_fragment:
- amazon.aws.common.modules
- amazon.aws.region.modules
- amazon.aws.boto3
"""

EXAMPLES = r"""
- name: list all the topics
  community.aws.sns_topic_info:
  register: sns_topic_list

- name: get info on specific topic
  community.aws.sns_topic_info:
    topic_arn: "{{ sns_arn }}"
  register: sns_topic_info
"""

RETURN = r"""
result:
  description:
    - The result contaning the details of one or all AWS SNS topics.
  returned: success
  type: list
  contains:
    sns_arn:
        description: The ARN of the topic.
        type: str
        returned: always
        sample: "arn:aws:sns:us-east-2:123456789012:my_topic_name"
    sns_topic:
        description: Dict of sns topic details.
        type: complex
        returned: always
        contains:
            content_based_deduplication:
              description: Whether or not content_based_deduplication was set
              returned: always
              type: str
              sample: "true"
            delivery_policy:
                description: Delivery policy for the SNS topic.
                returned: when topic is owned by this AWS account
                type: str
                sample: >
                    {"http":{"defaultHealthyRetryPolicy":{"minDelayTarget":20,"maxDelayTarget":20,"numRetries":3,"numMaxDelayRetries":0,
                    "numNoDelayRetries":0,"numMinDelayRetries":0,"backoffFunction":"linear"},"disableSubscriptionOverrides":false}}
            display_name:
                description: Display name for SNS topic.
                returned: when topic is owned by this AWS account
                type: str
                sample: My topic name
            owner:
                description: AWS account that owns the topic.
                returned: when topic is owned by this AWS account
                type: str
                sample: '123456789012'
            policy:
                description: Policy for the SNS topic.
                returned: when topic is owned by this AWS account
                type: str
                sample: >
                    {"Version":"2012-10-17","Id":"SomePolicyId","Statement":[{"Sid":"ANewSid","Effect":"Allow","Principal":{"AWS":"arn:aws:iam::123456789012:root"},
                    "Action":"sns:Subscribe","Resource":"arn:aws:sns:us-east-2:123456789012:ansible-test-dummy-topic","Condition":{"StringEquals":{"sns:Protocol":"email"}}}]}
            subscriptions:
                description: List of subscribers to the topic in this AWS account.
                returned: always
                type: list
                sample: []
            subscriptions_added:
                description: List of subscribers added in this run.
                returned: always
                type: list
                sample: []
            subscriptions_confirmed:
                description: Count of confirmed subscriptions.
                returned: when topic is owned by this AWS account
                type: str
                sample: '0'
            subscriptions_deleted:
                description: Count of deleted subscriptions.
                returned: when topic is owned by this AWS account
                type: str
                sample: '0'
            subscriptions_existing:
                description: List of existing subscriptions.
                returned: always
                type: list
                sample: []
            subscriptions_new:
                description: List of new subscriptions.
                returned: always
                type: list
                sample: []
            subscriptions_pending:
                description: Count of pending subscriptions.
                returned: when topic is owned by this AWS account
                type: str
                sample: '0'
            subscriptions_purge:
                description: Whether or not purge_subscriptions was set.
                returned: always
                type: bool
                sample: true
            topic_arn:
                description: ARN of the SNS topic (equivalent to sns_arn).
                returned: when topic is owned by this AWS account
                type: str
                sample: arn:aws:sns:us-east-2:123456789012:ansible-test-dummy-topic
            topic_type:
                description: The type of topic.
                type: str
                sample: "standard"
"""


try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.sns import get_info
from ansible_collections.community.aws.plugins.module_utils.sns import list_topics


def main():
    argument_spec = dict(
        topic_arn=dict(type="str", required=False),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    topic_arn = module.params.get("topic_arn")

    try:
        connection = module.client("sns", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    if topic_arn:
        results = dict(sns_arn=topic_arn, sns_topic=get_info(connection, module, topic_arn))
    else:
        results = list_topics(connection, module)

    module.exit_json(result=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: cloudfront_distribution_info
version_added: 1.0.0
short_description: Obtain facts about an AWS CloudFront distribution
description:
  - Gets information about an AWS CloudFront distribution.
  - Prior to release 5.0.0 this module was called C(community.aws.cloudfront_info).
    The usage did not change.
author:
  - Willem van Ketwich (@wilvk)
options:
    distribution_id:
        description:
          - The id of the CloudFront distribution. Used with I(distribution), I(distribution_config),
            I(invalidation), I(streaming_distribution), I(streaming_distribution_config), I(list_invalidations).
        required: false
        type: str
    invalidation_id:
        description:
          - The id of the invalidation to get information about.
          - Used with I(invalidation).
        required: false
        type: str
    origin_access_identity_id:
        description:
          - The id of the CloudFront origin access identity to get information about.
        required: false
        type: str
#    web_acl_id:
#        description:
#          - Used with I(list_distributions_by_web_acl_id).
#        required: false
#        type: str
    domain_name_alias:
        description:
          - Can be used instead of I(distribution_id) - uses the aliased CNAME for the CloudFront
            distribution to get the distribution id where required.
        required: false
        type: str
    all_lists:
        description:
          - Get all CloudFront lists that do not require parameters.
        required: false
        default: false
        type: bool
    origin_access_identity:
        description:
          - Get information about an origin access identity.
          - Requires I(origin_access_identity_id) to be specified.
        required: false
        default: false
        type: bool
    origin_access_identity_config:
        description:
          - Get the configuration information about an origin access identity.
          - Requires I(origin_access_identity_id) to be specified.
        required: false
        default: false
        type: bool
    distribution:
        description:
          - Get information about a distribution.
          - Requires I(distribution_id) or I(domain_name_alias) to be specified.
        required: false
        default: false
        type: bool
    distribution_config:
        description:
          - Get the configuration information about a distribution.
          - Requires I(distribution_id) or I(domain_name_alias) to be specified.
        required: false
        default: false
        type: bool
    invalidation:
        description:
          - Get information about an invalidation.
          - Requires I(invalidation_id) to be specified.
        required: false
        default: false
        type: bool
    streaming_distribution:
        description:
          - Get information about a specified RTMP distribution.
          - Requires I(distribution_id) or I(domain_name_alias) to be specified.
        required: false
        default: false
        type: bool
    streaming_distribution_config:
        description:
          - Get the configuration information about a specified RTMP distribution.
          - Requires I(distribution_id) or I(domain_name_alias) to be specified.
        required: false
        default: false
        type: bool
    list_origin_access_identities:
        description:
          - Get a list of CloudFront origin access identities.
          - Requires I(origin_access_identity_id) to be set.
        required: false
        default: false
        type: bool
    list_distributions:
        description:
          - Get a list of CloudFront distributions.
        required: false
        default: false
        type: bool
    list_distributions_by_web_acl_id:
        description:
          - Get a list of distributions using web acl id as a filter.
          - Requires I(web_acl_id) to be set.
        required: false
        default: false
        type: bool
    list_invalidations:
        description:
          - Get a list of invalidations.
          - Requires I(distribution_id) or I(domain_name_alias) to be specified.
        required: false
        default: false
        type: bool
    list_streaming_distributions:
        description:
          - Get a list of streaming distributions.
        required: false
        default: false
        type: bool
    summary:
        description:
          - Returns a summary of all distributions, streaming distributions and origin_access_identities.
          - This is the default behaviour if no option is selected.
        required: false
        default: false
        type: bool

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Get a summary of distributions
  community.aws.cloudfront_distribution_info:
    summary: true
  register: result

- name: Get information about a distribution
  community.aws.cloudfront_distribution_info:
    distribution: true
    distribution_id: my-cloudfront-distribution-id
  register: result_did
- ansible.builtin.debug:
    msg: "{{ result_did['cloudfront']['my-cloudfront-distribution-id'] }}"

- name: Get information about a distribution using the CNAME of the cloudfront distribution.
  community.aws.cloudfront_distribution_info:
    distribution: true
    domain_name_alias: www.my-website.com
  register: result_website
- ansible.builtin.debug:
    msg: "{{ result_website['cloudfront']['www.my-website.com'] }}"

- name: Get all information about an invalidation for a distribution.
  community.aws.cloudfront_distribution_info:
    invalidation: true
    distribution_id: my-cloudfront-distribution-id
    invalidation_id: my-cloudfront-invalidation-id

- name: Get all information about a CloudFront origin access identity.
  community.aws.cloudfront_distribution_info:
    origin_access_identity: true
    origin_access_identity_id: my-cloudfront-origin-access-identity-id

- name: Get all information about lists not requiring parameters (ie. list_origin_access_identities, list_distributions, list_streaming_distributions)
  community.aws.cloudfront_distribution_info:
    origin_access_identity: true
    origin_access_identity_id: my-cloudfront-origin-access-identity-id

- name: Get all information about lists not requiring parameters (ie. list_origin_access_identities, list_distributions, list_streaming_distributions)
  community.aws.cloudfront_distribution_info:
    all_lists: true
"""

RETURN = r"""
origin_access_identity:
    description: Describes the origin access identity information. Requires I(origin_access_identity_id) to be set.
    returned: only if I(origin_access_identity) is true
    type: dict
origin_access_identity_configuration:
    description: Describes the origin access identity information configuration information. Requires I(origin_access_identity_id) to be set.
    returned: only if I(origin_access_identity_configuration) is true
    type: dict
distribution:
    description: >
      Facts about a CloudFront distribution. Requires I(distribution_id) or I(domain_name_alias)
      to be specified. Requires I(origin_access_identity_id) to be set.
    returned: only if distribution is true
    type: dict
distribution_config:
    description: >
      Facts about a CloudFront distribution's config. Requires I(distribution_id) or I(domain_name_alias)
      to be specified.
    returned: only if I(distribution_config) is true
    type: dict
invalidation:
    description: >
      Describes the invalidation information for the distribution. Requires
      I(invalidation_id) to be specified and either I(distribution_id) or I(domain_name_alias.)
    returned: only if invalidation is true
    type: dict
streaming_distribution:
    description: >
      Describes the streaming information for the distribution. Requires
      I(distribution_id) or I(domain_name_alias) to be specified.
    returned: only if I(streaming_distribution) is true
    type: dict
streaming_distribution_config:
    description: >
      Describes the streaming configuration information for the distribution.
      Requires I(distribution_id) or I(domain_name_alias) to be specified.
    returned: only if I(streaming_distribution_config) is true
    type: dict
summary:
    description: Gives a summary of distributions, streaming distributions and origin access identities.
    returned: as default or if summary is true
    type: dict
result:
    description: >
        Result dict not nested under the CloudFront ID to access results of module without the knowledge of that id
        as figuring out the DistributionId is usually the reason one uses this module in the first place.
    returned: always
    type: dict
"""

from ansible_collections.amazon.aws.plugins.module_utils.cloudfront_facts import CloudFrontFactsServiceManager

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def set_facts_for_distribution_id_and_alias(details, facts, distribution_id, aliases):
    facts[distribution_id] = details
    # also have a fixed key for accessing results/details returned
    facts["result"] = details
    facts["result"]["DistributionId"] = distribution_id

    for alias in aliases:
        facts[alias] = details
    return facts


def main():
    argument_spec = dict(
        distribution_id=dict(required=False, type="str"),
        invalidation_id=dict(required=False, type="str"),
        origin_access_identity_id=dict(required=False, type="str"),
        domain_name_alias=dict(required=False, type="str"),
        all_lists=dict(required=False, default=False, type="bool"),
        distribution=dict(required=False, default=False, type="bool"),
        distribution_config=dict(required=False, default=False, type="bool"),
        origin_access_identity=dict(required=False, default=False, type="bool"),
        origin_access_identity_config=dict(required=False, default=False, type="bool"),
        invalidation=dict(required=False, default=False, type="bool"),
        streaming_distribution=dict(required=False, default=False, type="bool"),
        streaming_distribution_config=dict(required=False, default=False, type="bool"),
        list_origin_access_identities=dict(required=False, default=False, type="bool"),
        list_distributions=dict(required=False, default=False, type="bool"),
        list_distributions_by_web_acl_id=dict(required=False, default=False, type="bool"),
        list_invalidations=dict(required=False, default=False, type="bool"),
        list_streaming_distributions=dict(required=False, default=False, type="bool"),
        summary=dict(required=False, default=False, type="bool"),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    service_mgr = CloudFrontFactsServiceManager(module)

    distribution_id = module.params.get("distribution_id")
    invalidation_id = module.params.get("invalidation_id")
    origin_access_identity_id = module.params.get("origin_access_identity_id")
    web_acl_id = module.params.get("web_acl_id")
    domain_name_alias = module.params.get("domain_name_alias")
    all_lists = module.params.get("all_lists")
    distribution = module.params.get("distribution")
    distribution_config = module.params.get("distribution_config")
    origin_access_identity = module.params.get("origin_access_identity")
    origin_access_identity_config = module.params.get("origin_access_identity_config")
    invalidation = module.params.get("invalidation")
    streaming_distribution = module.params.get("streaming_distribution")
    streaming_distribution_config = module.params.get("streaming_distribution_config")
    list_origin_access_identities = module.params.get("list_origin_access_identities")
    list_distributions = module.params.get("list_distributions")
    list_distributions_by_web_acl_id = module.params.get("list_distributions_by_web_acl_id")
    list_invalidations = module.params.get("list_invalidations")
    list_streaming_distributions = module.params.get("list_streaming_distributions")
    summary = module.params.get("summary")

    aliases = []
    result = {"cloudfront": {}}
    facts = {}

    require_distribution_id = (
        distribution
        or distribution_config
        or invalidation
        or streaming_distribution
        or streaming_distribution_config
        or list_invalidations
    )

    # set default to summary if no option specified
    summary = summary or not (
        distribution
        or distribution_config
        or origin_access_identity
        or origin_access_identity_config
        or invalidation
        or streaming_distribution
        or streaming_distribution_config
        or list_origin_access_identities
        or list_distributions_by_web_acl_id
        or list_invalidations
        or list_streaming_distributions
        or list_distributions
    )

    # validations
    if require_distribution_id and distribution_id is None and domain_name_alias is None:
        module.fail_json(msg="Error distribution_id or domain_name_alias have not been specified.")
    if invalidation and invalidation_id is None:
        module.fail_json(msg="Error invalidation_id has not been specified.")
    if (origin_access_identity or origin_access_identity_config) and origin_access_identity_id is None:
        module.fail_json(msg="Error origin_access_identity_id has not been specified.")
    if list_distributions_by_web_acl_id and web_acl_id is None:
        module.fail_json(msg="Error web_acl_id has not been specified.")

    # get distribution id from domain name alias
    if require_distribution_id and distribution_id is None:
        distribution_id = service_mgr.get_distribution_id_from_domain_name(domain_name_alias)
        if not distribution_id:
            module.fail_json(msg="Error unable to source a distribution id from domain_name_alias")

    # set appropriate cloudfront id
    if invalidation_id is not None and invalidation:
        facts.update({invalidation_id: {}})
    if origin_access_identity_id and (origin_access_identity or origin_access_identity_config):
        facts.update({origin_access_identity_id: {}})
    if web_acl_id:
        facts.update({web_acl_id: {}})

    # get details based on options
    if distribution:
        facts_to_set = service_mgr.get_distribution(id=distribution_id)
    if distribution_config:
        facts_to_set = service_mgr.get_distribution_config(id=distribution_id)
    if origin_access_identity:
        facts[origin_access_identity_id].update(service_mgr.get_origin_access_identity(id=origin_access_identity_id))
    if origin_access_identity_config:
        facts[origin_access_identity_id].update(
            service_mgr.get_origin_access_identity_config(id=origin_access_identity_id)
        )
    if invalidation:
        facts_to_set = service_mgr.get_invalidation(distribution_id=distribution_id, id=invalidation_id)
        facts[invalidation_id].update(facts_to_set)
    if streaming_distribution:
        facts_to_set = service_mgr.get_streaming_distribution(id=distribution_id)
    if streaming_distribution_config:
        facts_to_set = service_mgr.get_streaming_distribution_config(id=distribution_id)
    if list_invalidations:
        invalidations = service_mgr.list_invalidations(distribution_id=distribution_id) or {}
        facts_to_set = {"invalidations": invalidations}
    if "facts_to_set" in vars():
        aliases = service_mgr.get_aliases_from_distribution_id(distribution_id)
        facts = set_facts_for_distribution_id_and_alias(facts_to_set, facts, distribution_id, aliases)

    # get list based on options
    if all_lists or list_origin_access_identities:
        facts["origin_access_identities"] = service_mgr.list_origin_access_identities() or {}
    if all_lists or list_distributions:
        facts["distributions"] = service_mgr.list_distributions() or {}
    if all_lists or list_streaming_distributions:
        facts["streaming_distributions"] = service_mgr.list_streaming_distributions() or {}
    if list_distributions_by_web_acl_id:
        facts["distributions_by_web_acl_id"] = service_mgr.list_distributions_by_web_acl_id(web_acl_id=web_acl_id) or {}
    if list_invalidations:
        facts["invalidations"] = service_mgr.list_invalidations(distribution_id=distribution_id) or {}

    # default summary option
    if summary:
        facts["summary"] = service_mgr.summary()

    result["changed"] = False
    result["cloudfront"].update(facts)

    module.exit_json(msg="Retrieved CloudFront info.", **result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_organization
short_description: Organization configuration for Cisco Intersight
description:
  - Manages Organization configuration on Cisco Intersight.
  - Organizations are account-level resources used to organize and manage Intersight resources.
  - Supports assigning Resource Groups to define resource scope.
  - Supports sharing resources with other Organizations via sharing rules.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/organization/Organization/model/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  name:
    description:
      - The name assigned to the Organization.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Organization.
    type: str
    aliases: [descr]
  share_resources:
    description:
      - Enable this option to share resources with other Organizations.
      - When C(true), use O(shared_orgs) to specify which Organizations to share with.
      - Mutually exclusive with O(resource_groups).
    type: bool
    default: false
  resource_groups:
    description:
      - List of Resource Group names to assign to the Organization.
      - Resource Group names are resolved to their Moids automatically.
      - Mutually exclusive with O(share_resources) and O(shared_orgs).
    type: list
    elements: str
  shared_orgs:
    description:
      - List of Organization names to share resources with.
      - Only valid when O(share_resources) is C(true).
      - Organization names are resolved to their Moids automatically.
    type: list
    elements: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Organization with Resource Groups
  cisco.intersight.intersight_organization:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "my-organization"
    description: "My organization"
    resource_groups:
      - "my-resource-group"
    state: present

- name: Create Organization with shared resources
  cisco.intersight.intersight_organization:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "shared-org"
    description: "Organization sharing resources"
    share_resources: true
    shared_orgs:
      - "other-org-1"
      - "other-org-2"
    state: present

- name: Create Organization without Resource Groups
  cisco.intersight.intersight_organization:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "simple-org"
    description: "Simple organization"
    state: present

- name: Delete Organization
  cisco.intersight.intersight_organization:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "my-organization"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "my-organization",
        "ObjectType": "organization.Organization",
        "Description": "My organization"
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def build_api_body(intersight):
    """Build the API body for Organization configuration."""
    params = intersight.module.params
    if params['state'] == 'present':
        intersight.api_body = {
            'Name': params['name'],
        }

        if not params['share_resources']:
            if params.get('resource_groups'):
                rg_refs = []
                for rg_name in params['resource_groups']:
                    moid = intersight.get_moid_by_name('/resource/Groups', rg_name)
                    if moid is None:
                        intersight.module.fail_json(msg=f"Resource Group '{rg_name}' not found")
                    rg_refs.append({'ObjectType': 'resource.Group', 'Moid': moid})
                intersight.api_body['ResourceGroups'] = rg_refs
            else:
                intersight.api_body['ResourceGroups'] = []
        else:
            # Shared mode: clear any previously assigned ResourceGroups
            intersight.api_body['ResourceGroups'] = []

        if params.get('description') is not None:
            intersight.api_body['Description'] = params['description']


def get_sharing_rules(intersight, org_moid):
    """Get current sharing rules for an organization."""
    intersight.get_resource(
        resource_path='/iam/SharingRules',
        query_params={
            '$filter': f"SharedResource.Moid eq '{org_moid}'",
        },
        return_list=True,
    )
    return intersight.result['api_response'] if isinstance(intersight.result['api_response'], list) else []


def configure_sharing_rules(intersight, org_moid):
    """Configure sharing rules for an organization."""
    params = intersight.module.params
    shared_org_names = params.get('shared_orgs') or []

    # Get current sharing rules
    current_rules = get_sharing_rules(intersight, org_moid)

    # Build map of current shared org Moids to rule Moids
    current_shared = {}
    for rule in current_rules:
        shared_with_moid = rule.get('SharedWithResource', {}).get('Moid')
        if shared_with_moid:
            current_shared[shared_with_moid] = rule['Moid']

    # Resolve desired shared org names to Moids
    desired_shared = {}
    for org_name in shared_org_names:
        moid = intersight.get_moid_by_name('/organization/Organizations', org_name)
        if moid is None:
            intersight.module.fail_json(msg=f"Organization '{org_name}' not found")
        desired_shared[moid] = org_name

    # Compute diff
    current_moids = set(current_shared.keys())
    desired_moids = set(desired_shared.keys())
    to_add = desired_moids - current_moids
    to_remove = current_moids - desired_moids

    # Delete removed sharing rules
    if to_remove:
        delete_data = [{'moid': current_shared[moid]} for moid in to_remove]
        intersight.configure_bulk_resources(
            resource_path='/iam/SharingRules',
            resources_data=delete_data,
            http_method='delete',
        )

    # Create new sharing rules
    if to_add:
        create_data = [
            {
                'body': {
                    'SharedResource': {
                        'ObjectType': 'organization.Organization',
                        'Moid': org_moid,
                    },
                    'SharedWithResource': {
                        'ObjectType': 'organization.Organization',
                        'Moid': shared_moid,
                    },
                }
            }
            for shared_moid in to_add
        ]
        intersight.configure_bulk_resources(
            resource_path='/iam/SharingRules',
            resources_data=create_data,
            http_method='post',
        )


def cleanup_sharing_rules(intersight, org_moid):
    """Remove all sharing rules for an organization."""
    current_rules = get_sharing_rules(intersight, org_moid)
    if current_rules:
        delete_data = [{'moid': rule['Moid']} for rule in current_rules]
        intersight.configure_bulk_resources(
            resource_path='/iam/SharingRules',
            resources_data=delete_data,
            http_method='delete',
        )


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        share_resources=dict(type='bool', default=False),
        resource_groups=dict(type='list', elements='str'),
        shared_orgs=dict(type='list', elements='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        required_if=[
            ['share_resources', True, ['shared_orgs']],
        ],
    )

    if module.params['share_resources'] and module.params.get('resource_groups'):
        module.fail_json(msg="parameters are mutually exclusive: share_resources=true and resource_groups")

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    build_api_body(intersight)

    resource_path = '/organization/Organizations'

    # Clean up sharing rules BEFORE modifying the organization.
    # The Intersight API rejects ResourceGroup changes on orgs that still
    # have active sharing rules, so removal must happen first.
    if not module.check_mode:
        intersight.get_resource(
            resource_path=resource_path,
            query_params={'$filter': f"Name eq '{module.params['name']}'"},
        )
        existing_moid = intersight.result['api_response'].get('Moid')
        if existing_moid and module.params['state'] == 'absent' or not module.params['share_resources']:
            cleanup_sharing_rules(intersight, existing_moid)

    intersight.configure_secondary_resource(
        resource_path=resource_path,
        resource_name=module.params['name'],
        state=module.params['state'],
    )

    org_response = intersight.result['api_response']
    org_moid = org_response.get('Moid')

    if module.params['state'] == 'present' and org_moid:
        if module.params['share_resources']:
            configure_sharing_rules(intersight, org_moid)

    intersight.result['api_response'] = org_response
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_vnic_template_info
short_description: Gather information about vNIC Templates in Cisco Intersight
description:
  - Retrieve comprehensive information about vNIC Templates from L(Cisco Intersight,https://intersight.com).
  - Query templates by organization, template name, or other filters.
  - Returns structured data with template metadata and policy associations.
  - If no filters are provided, all vNIC Templates will be returned.
  - vNIC Templates are used to define standardized network interface configurations for FI-Attached deployments.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization to filter vNIC Templates by.
      - Use 'default' for the default organization.
      - When specified, only templates from this organization will be returned.
    type: str
  name:
    description:
      - The exact name of the vNIC Template to retrieve information from.
      - When specified, only the matching template will be returned.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
# Basic Usage Examples
- name: Fetch all vNIC Templates from all organizations
  cisco.intersight.intersight_vnic_template_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
  register: all_vnic_templates

- name: Display all template names
  debug:
    msg: "Template: {{ item.Name }}"
  loop: "{{ all_vnic_templates.api_response }}"
  when: all_vnic_templates.api_response is iterable

# Organization-specific Examples
- name: Fetch all vNIC Templates from the default organization
  cisco.intersight.intersight_vnic_template_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
  register: default_org_templates

- name: Fetch all vNIC Templates from a custom organization
  cisco.intersight.intersight_vnic_template_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
  register: engineering_templates
'''

RETURN = r'''
api_response:
  description:
    - The API response containing vNIC Template information.
    - Returns a dictionary when querying a single template or no templates found.
    - Returns a list when multiple templates are found.
  returned: always
  type: dict
  sample:
    Name: "production-vnic-template"
    ObjectType: "vnic.VnicTemplate"
    Moid: "12345678901234567890abcd"
    Description: "Production vNIC template for FI-attached servers"
    EnableOverride: false
    SwitchId: "A"
    FailoverEnabled: false
    Cdn:
      Source: "vnic"
    MacPool:
      Name: "default-mac-pool"
      ObjectType: "macpool.Pool"
      Moid: "macpool12345678901234567890"
    FabricEthNetworkGroupPolicy:
      - Name: "default-network-group"
        ObjectType: "fabric.EthNetworkGroupPolicy"
        Moid: "netgroup12345678901234567890"
    FabricEthNetworkControlPolicy:
      Name: "default-network-control"
      ObjectType: "fabric.EthNetworkControlPolicy"
      Moid: "netcontrol12345678901234567890"
    EthQosPolicy:
      Name: "default-qos-policy"
      ObjectType: "vnic.EthQosPolicy"
      Moid: "qos12345678901234567890"
    EthAdapterPolicy:
      Name: "default-adapter-policy"
      ObjectType: "vnic.EthAdapterPolicy"
      Moid: "adapter12345678901234567890"
    Organization:
      Name: "default"
      ObjectType: "organization.Organization"
      Moid: "org12345678901234567890"
    Tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    SriovSettings:
      Enabled: true
      VfCount: 64
      RxCountPerVf: 4
      TxCountPerVf: 1
      CompCountPerVf: 5
      IntCountPerVf: 8
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)

    # Resource path used to fetch vNIC Template info
    resource_path = '/vnic/VnicTemplates'

    # Get query parameters for templates
    query_params = intersight.set_query_params()

    # Reset api_response before the API call to avoid previous responses
    intersight.result['api_response'] = {}

    # Get vNIC Templates
    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    templates = intersight.result.get('api_response', [])

    # Create final response structure
    final_api_response = None

    # Ensure templates is always a list for checking length, even if a single dict is returned
    if isinstance(templates, dict):
        templates = [templates]
    elif not isinstance(templates, list):
        templates = []

    # Set final response based on number of templates found
    if len(templates) == 1:
        # Single template - return as dict
        final_api_response = templates[0]
    elif len(templates) > 1:
        # Multiple templates - return as list
        final_api_response = templates
    else:
        # No templates found - return empty dict
        final_api_response = {}

    # Use intersight.result and update api_response directly
    intersight.result['api_response'] = final_api_response
    final_result = intersight.result

    module.exit_json(**final_result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, John Garland (@johnnyg) <johnnybg@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: clickhouse_quota

short_description: Creates or removes a ClickHouse quota

description:
  - Creates or removes a ClickHouse quota.

attributes:
  check_mode:
    description: Supports check_mode.
    support: full
  idempotent:
    description: At second run will not change anything.
    support: full

version_added: '1.1.0'

author:
  - John Garland (@johnnyg)

extends_documentation_fragment:
  - community.clickhouse.client_inst_opts

options:
  state:
    description:
      - Quota state.
      - V(present) creates the quota if it does not exist.
      - V(absent) deletes the quota if it exists.
    type: str
    choices: ['present', 'absent']
    default: 'present'
  name:
    description:
      - Quota name to add or remove.
    type: str
    required: true
  cluster:
    description:
      - Run the command on all cluster hosts.
      - If the cluster is not configured, the command will crash with an error.
    type: str
  keyed_by:
    description:
      - Keys the quota by the specified key (default is to not key).
    type: str
    choices:
      - user_name
      - ip_address
      - client_key
      - client_key,user_name
      - client_key,ip_address
  limits:
    description:
      - The limits that this quota should enforce.
    type: list
    elements: dict
    suboptions:
      randomized_start:
        description:
          - Whether this interval's start should be randomized.
          - Intervals always start at the same time if not randomized.
        type: bool
        default: false
      interval:
        description:
          - The interval to apply the following quotas on.
          - This is in the format C(<number> <unit>).
          - Where unit is one of second, minute, hour, day, week, month, quarter or year.
        type: str
        required: true
      max:
        description:
          - Maximum values to apply to this interval in this quota.
          - At least one key must be specified.
          - Mutually exclusive with O(limits.no_limits) and O(limits.tracking_only).
        type: dict
        suboptions:
          queries:
            description:
              - Maximum number of queries to enforce in this interval.
            type: int
          query_selects:
            description:
              - Maximum number of query selects to enforce in this interval.
            type: int
          query_inserts:
            description:
              - Maximum number of query inserts to enforce in this interval.
            type: int
          errors:
            description:
              - Maximum number of errors to enforce in this interval.
            type: int
          result_rows:
            description:
              - Maximum number of result rows to enforce in this interval.
            type: int
          result_bytes:
            description:
              - Maximum number of result bytes to enforce in this interval.
            type: int
          read_rows:
            description:
              - Maximum number of rows read to enforce in this interval.
            type: int
          read_bytes:
            description:
              - Maximum number of bytes read to enforce in this interval.
            type: int
          written_bytes:
            description:
              - Maximum number of bytes written to enforce in this interval.
            type: int
          execution_time:
            description:
              - Maximum number of execution time to enforce in this interval.
            type: float
          failed_sequential_authentications:
            description:
              - Maximum number of failed sequential authentications to enforce in this interval.
            type: int
      no_limits:
        description:
          - Don't apply any limits.
          - Mutually exclusive with O(limits.max) and O(limits.tracking_only).
        type: bool
      tracking_only:
        description:
          - Just track usage instead of enforcing.
          - Mutually exclusive with O(limits.max) and O(limits.no_limits).
        type: bool
  apply_to:
    description:
      - Apply this quota to the following list of users/roles dependent on O(apply_to_mode).
      - Can include special keywords of default and current_user or the name of an actual user or role.
      - Is an error to specify this if O(apply_to_mode=all).
    type: list
    elements: str
  apply_to_mode:
    description:
      - When V(listed_only) (default), the quota will only apply to the users/roles specified in O(apply_to).
      - When V(all), the quota will only apply to B(all) users/roles.
      - When V(all_except_listed), the quota will only apply to B(all) the users/roles except those specified in O(apply_to).
    type: str
    choices: ['listed_only', 'all', 'all_except_listed']
    default: 'listed_only'
"""

EXAMPLES = r"""
- name: Create quota
  community.clickhouse.clickhouse_quota:
    name: test_quota
    state: present

- name: Create a quota with limits
  community.clickhouse.clickhouse_quota:
    name: test_quota
    state: present
    limits:
      - interval: 5 minute
        max:
          queries: 100
          execution_time: 100
        apply_to:
          - one_role
          - another_role
    cluster: test_cluster

- name: Remove quota
  community.clickhouse.clickhouse_quota:
    name: test_quota
    state: absent
"""

RETURN = r"""
executed_statements:
  description:
  - Data-modifying executed statements.
  returned: on success
  type: list
  sample: ['CREATE QUOTA test_quota']
"""

import copy
import re
from operator import itemgetter

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.clickhouse.plugins.module_utils.clickhouse import (
    check_clickhouse_driver,
    client_common_argument_spec,
    connect_to_db_via_client,
    execute_query,
    get_main_conn_kwargs,
)

_VALID_NAME_REGEX = re.compile(r"^[^'\"`;\0]+$")
_POSSIBLY_ESCAPED_NAME_REGEX = r"(?:`(?:[^`]+)`)|(?:\w+)"
_KEYED_BY_VALUES = [
    "user_name",
    "ip_address",
    "client_key, ?user_name",
    "client_key, ?ip_address",
    "client_key",  # needs to be after so it doesn't match first
]
_CREATE_QUOTA_REGEX = re.compile(
    rf"^CREATE QUOTA (?P<name>{_POSSIBLY_ESCAPED_NAME_REGEX})"
    rf"(?: ON CLUSTER (?P<cluster>{_POSSIBLY_ESCAPED_NAME_REGEX}))?"
    rf"(?: KEYED BY (?P<keyed_by>{'|'.join(_KEYED_BY_VALUES)}))?"
)

_NUMBER_REGEX = r"(?:-?\d+\.?\d*)"
_INTERVAL_UNITS = [
    "second",
    "minute",
    "hour",
    "day",
    "week",
    "month",
    "quarter",
    "year",
]
_MAX_LIMIT_TYPES = [
    "queries",
    "query_selects",
    "query_inserts",
    "errors",
    "result_rows",
    "result_bytes",
    "read_rows",
    "read_bytes",
    "written_bytes",
    "execution_time",
    "failed_sequential_authentications",
]
_LIMIT_TYPES = [
    rf"(?:MAX(?:,? (?:{'|'.join(_MAX_LIMIT_TYPES)}) = {_NUMBER_REGEX})+)",
    "NO LIMITS",
    "TRACKING ONLY",
]
_LIMITS_REGEX = re.compile(
    r"FOR (?:(?P<randomized>RANDOMIZED) )?"
    rf"INTERVAL (?P<interval_number>{_NUMBER_REGEX}) (?P<interval_unit>{'|'.join(_INTERVAL_UNITS)}) "
    rf"(?P<limit_type>{'|'.join(_LIMIT_TYPES)})"
)

_ROLES_REGEX = rf"(?:(?:{_POSSIBLY_ESCAPED_NAME_REGEX})(?:, )?)+"
_APPLY_TO_TYPES = [rf"(?:{_ROLES_REGEX})", "ALL", rf"ALL EXCEPT (?:{_ROLES_REGEX})"]
_APPLY_TO_REGEX = re.compile(rf" TO (?P<apply_to>{'|'.join(_APPLY_TO_TYPES)})$")
_USER_OR_ROLE_REGEX = re.compile(rf"(?P<name>{_POSSIBLY_ESCAPED_NAME_REGEX})(?:, ?)?")


_DEFAULT_LIMIT_PARAMS = {
    "randomized_start": False,
    "interval": None,
    "max": {},
    "no_limits": None,
    "tracking_only": None,
}

_DEFAULT_MAX_PARAMS = {
    "queries": None,
    "query_selects": None,
    "query_inserts": None,
    "errors": None,
    "result_rows": None,
    "result_bytes": None,
    "read_rows": None,
    "read_bytes": None,
    "written_bytes": None,
    "execution_time": None,
    "failed_sequential_authentications": None,
}

_DEFAULT_PARAMS = {
    "cluster": None,
    "keyed_by": None,
    "limits": [],
    "apply_to": [],
    "apply_to_mode": "listed_only",
}


class ClickHouseQuota:
    _type = "QUOTA"

    def __init__(self, module, client, name):
        if not _VALID_NAME_REGEX.match(name):
            raise ValueError(f"'{name}' is not a valid quota name")
        self.module = module
        self.client = client
        self.name = name
        self.executed_statements = []
        self.exists = self._check_exists()

    def _check_exists(self):
        query = f"SELECT 1 FROM system.{self._type.lower()}s WHERE name = '{self.name}' LIMIT 1"
        result = execute_query(self.module, self.client, query)
        return bool(result)

    def _get_create_statement(self):
        """Get current definition using SHOW CREATE X"""
        if not self.exists:
            return None

        query = f"SHOW CREATE {self._type} '{self.name}'"
        result = execute_query(self.module, self.client, query)
        if result:
            # SHOW CREATE X returns single row with CREATE statement
            return result[0][0]
        return None

    def _needs_altering(self):
        """Check if we need to alter to reach desired"""
        create_statement = self._get_create_statement()
        if create_statement is None:
            return True
        current_params = self._normalize(self._parse_create_statement(create_statement))
        desired_params = self._normalize(self.module.params)

        # For debugging version compatibility issues
        if self.module._verbosity >= 3:  # Only show at high verbosity
            self.module.log(f"Current params (normalized): {current_params}")
            self.module.log(f"Desired params (normalized): {desired_params}")

        return current_params != desired_params

    def _do(self, action):
        if action not in ("CREATE", "ALTER"):
            raise ValueError(
                f"Expected action to be CREATE or ALTER but got '{action}'"
            )

        query = " ".join(self._create_sql_clauses(action))

        self.executed_statements.append(query)

        if not self.module.check_mode:
            execute_query(self.module, self.client, query)

    def create(self):
        """
        Create entity using CREATE X
        Returns whether the entity was created or not
        """
        if self.exists:
            return False

        self._do("CREATE")
        self.exists = True
        return True

    def alter(self):
        """
        Update entity using ALTER X if it needs it
        Returns whether the entity was altered or not
        """
        if not self.exists or not self._needs_altering():
            return False

        self._do("ALTER")
        return True

    def drop(self):
        """Drop entity using DROP X"""
        if not self.exists:
            return False

        query = f"DROP {self._type} '{self.name}'"
        self.executed_statements.append(query)

        if not self.module.check_mode:
            execute_query(self.module, self.client, query)

        self.exists = False
        return True

    def ensure_state(self):
        state = self.module.params["state"]
        if state not in ("present", "absent"):
            raise ValueError(f"Unexpected state '{state}'")

        if state == "present":
            # create or alter role
            # will do nothing is nothing needs to be done
            changed = self.create() or self.alter()
        else:
            # drop if exists
            changed = self.drop()

        return changed

    @staticmethod
    def _parse_create_statement(create_statement):
        match = _CREATE_QUOTA_REGEX.match(create_statement)
        if not match:
            raise ValueError(f"Could not parse '{create_statement}'")
        params = {} | match.groupdict()
        params.pop("name")
        cluster = params["cluster"]
        if cluster is not None:
            params["cluster"] = cluster.strip("`")
        next_search_pos = match.end()
        limits = []
        for match in _LIMITS_REGEX.finditer(create_statement, pos=next_search_pos):
            limit = {}
            groups = match.groupdict()
            limit["randomized_start"] = groups["randomized"] == "RANDOMIZED"
            limit["interval"] = f"{groups['interval_number']} {groups['interval_unit']}"
            limit_type = groups["limit_type"]
            if limit_type == "NO LIMITS":
                limit["no_limits"] = True
            elif limit_type == "TRACKING ONLY":
                limit["tracking_only"] = True
            elif limit_type.startswith("MAX "):
                max_limits = {}
                for max_limit in limit_type[len("MAX ") :].split(", "):
                    key, _part, value = max_limit.partition(" = ")
                    type_fn = float if key == "execution_time" else int
                    max_limits[key] = type_fn(value)
                limit["max"] = max_limits
            else:
                raise ValueError(f"Invalid limit type '{limit_type}'")
            limits.append(limit)
            next_search_pos = match.end()
        params["limits"] = limits
        match = _APPLY_TO_REGEX.match(create_statement, pos=next_search_pos)
        if match:
            groups = match.groupdict()
            apply_to = groups["apply_to"]
            if apply_to == "ALL":
                params["apply_to_mode"] = "all"
                apply_to = ""
            elif apply_to.startswith("ALL EXCEPT "):
                params["apply_to_mode"] = "all_except_listed"
                apply_to = apply_to[len("ALL EXCEPT ") :]
            else:
                params["apply_to_mode"] = "listed_only"
            params["apply_to"] = [
                match.groupdict()["name"].strip("`")
                for match in _USER_OR_ROLE_REGEX.finditer(apply_to)
            ]
        return params

    @staticmethod
    def _normalize(params):
        normalized = _DEFAULT_PARAMS.copy()
        for key in normalized.keys() & params.keys():
            value = params[key]
            if value is not None:
                if key == "limits":
                    normalized_limits = []
                    for limit_params in value:
                        normalized_limit = _DEFAULT_LIMIT_PARAMS.copy()
                        for limit_key in normalized_limit.keys() & limit_params.keys():
                            limit_value = limit_params[limit_key]
                            if limit_value is not None:
                                normalized_limit[limit_key] = limit_value
                        normalized_limits.append(normalized_limit)
                    normalized[key] = normalized_limits
                else:
                    normalized[key] = copy.deepcopy(value)
        keyed_by = normalized["keyed_by"]
        if keyed_by:
            normalized["keyed_by"] = ",".join(
                key.strip() for key in keyed_by.split(",")
            )
        if (
            normalized["apply_to_mode"] == "all_except_listed"
            and not normalized["apply_to"]
        ):
            normalized["apply_to_mode"] = "all"
        # no limits is the default so they automatically get removed
        normalized["limits"] = [
            limit for limit in normalized["limits"] if not limit.get("no_limits")
        ]
        for limit in normalized["limits"]:
            max_limit = limit["max"]
            if max_limit:
                limit["max"] = _DEFAULT_MAX_PARAMS | max_limit
        normalized["limits"].sort(key=itemgetter("interval"))
        normalized["apply_to"].sort()
        return normalized

    def _create_sql_clauses(self, action):
        sql_clauses = [f"{action} {self._type} '{self.name}'"]

        cluster = self.module.params["cluster"]
        if cluster:
            sql_clauses.append(f"ON CLUSTER '{cluster}'")

        keyed_by = self.module.params.get("keyed_by")
        if keyed_by:
            sql_clauses.append(f"KEYED BY {keyed_by}")

        limits_sql_clauses = []
        for limit in self.module.params["limits"] or []:
            sql_clause = ["FOR"]
            if limit.get("randomized_start", False):
                sql_clause.append("RANDOMIZED")
            sql_clause.append(f"INTERVAL {limit['interval']}")
            max_limits = {
                key: value
                for key, value in (limit.get("max") or {}).items()
                if value is not None
            }
            if max_limits:
                sql_clause.append("MAX")
                sql_clause.append(
                    ", ".join(f"{key} = {value}" for key, value in max_limits.items())
                )
            elif limit.get("no_limits"):
                sql_clause.append("NO LIMITS")
            elif limit.get("tracking_only"):
                sql_clause.append("TRACKING ONLY")
            else:
                raise ValueError(
                    "One of max or no_limits or tracking_only needs to specified"
                )
            limits_sql_clauses.append(" ".join(sql_clause))
        if limits_sql_clauses:
            sql_clauses.append(", ".join(limits_sql_clauses))

        apply_to = self.module.params.get("apply_to", [])
        apply_to_mode = self.module.params["apply_to_mode"]
        if apply_to_mode == "all_except_listed" and not apply_to:
            apply_to_mode = "all"
        if apply_to and apply_to_mode == "all":
            raise ValueError(
                "Cannot specify list of user/roles to apply to when apply_to_mode == all"
            )
        if apply_to_mode == "all":
            sql_clauses.append("TO ALL")
        elif apply_to:
            sql_clauses.append("TO")
            if apply_to_mode == "all_except_listed":
                sql_clauses.append("ALL EXCEPT")
            sql_clauses.append(", ".join(apply_to))

        return sql_clauses


def main():
    # Set up arguments.
    # If there are common arguments shared across several modules,
    # create the common_argument_spec() function under plugins/module_utils/*
    # and invoke here to return a dict with those arguments
    argument_spec = client_common_argument_spec()
    argument_spec.update(
        state=dict(type="str", choices=["present", "absent"], default="present"),
        name=dict(type="str", required=True),
        cluster=dict(type="str", default=None),
        keyed_by=dict(
            type="str",
            choices=[
                "user_name",
                "ip_address",
                "client_key",
                "client_key,user_name",
                "client_key,ip_address",
            ],
        ),
        limits=dict(
            type="list",
            elements="dict",
            options=dict(
                randomized_start=dict(type="bool", default=False),
                interval=dict(type="str", required=True),
                max=dict(
                    type="dict",
                    options=dict(
                        queries=dict(type="int"),
                        query_selects=dict(type="int"),
                        query_inserts=dict(type="int"),
                        errors=dict(type="int"),
                        result_rows=dict(type="int"),
                        result_bytes=dict(type="int"),
                        read_rows=dict(type="int"),
                        read_bytes=dict(type="int"),
                        written_bytes=dict(type="int"),
                        execution_time=dict(type="float"),
                        failed_sequential_authentications=dict(type="int"),
                    ),
                    required_one_of=[
                        (
                            "queries",
                            "query_selects",
                            "query_inserts",
                            "errors",
                            "result_rows",
                            "result_bytes",
                            "read_rows",
                            "read_bytes",
                            "written_bytes",
                            "execution_time",
                            "failed_sequential_authentications",
                        )
                    ],
                ),
                no_limits=dict(type="bool"),
                tracking_only=dict(type="bool"),
            ),
            mutually_exclusive=[("max", "no_limits", "tracking_only")],
            required_one_of=[("max", "no_limits", "tracking_only")],
        ),
        apply_to=dict(type="list", elements="str"),
        apply_to_mode=dict(
            type="str",
            choices=["listed_only", "all", "all_except_listed"],
            default="listed_only",
        ),
    )

    # Instantiate an object of module class
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    # Assign passed options to variables
    client_kwargs = module.params["client_kwargs"]
    # The reason why these arguments are separate from client_kwargs
    # is that we need to protect some sensitive data like passwords passed
    # to the module from logging (see the arguments above with no_log=True);
    # Such data must be passed as module arguments (not nested deep in values).
    main_conn_kwargs = get_main_conn_kwargs(module)
    name = module.params["name"]

    # Will fail if no driver informing the user
    check_clickhouse_driver(module)

    # Connect to DB
    client = connect_to_db_via_client(module, main_conn_kwargs, client_kwargs)

    # Do the job
    quota = ClickHouseQuota(module, client, name)
    changed = quota.ensure_state()

    # Close connection
    client.disconnect_connection()

    # Users will get this in JSON output after execution
    module.exit_json(changed=changed, executed_statements=quota.executed_statements)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistryscopemap
version_added: "3.7.0"
short_description: Managed the Azure Container Registry Scope Map
description:
    - Create, update or delete the Container Registry Scope Map.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        type: str
        required: true
    registry_name:
        description:
            - The name of the container registry.
        type: str
        required: true
    name:
        description:
            - The name of the container registry scope map.
        type: str
        required: true
    description:
        description:
            - The user friendly description of the scope map.
        type: str
    actions:
        description:
            - The list of scoped permissions for registry artifacts.
            - Sample as C(repositories/repository-name/content/read) or C(repositories/repository-name/metadata/write).
        type: list
        elements: str
    state:
        description:
            - Assert the state of the container registry scope map.
            - Use C(present) to create or update a server and C(absent) to delete it.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get instance of Registry Scope Map
  azure_rm_containerregistryscopemap:
    resource_group: myResourceGroup
    registry_name: myRegistry
    name: myscopemap

- name: Delete the container registry scope map
  azure_rm_containerregistryscopemap:
    resource_group: myResourceGroup
    registry_name: myRegistry
    name: myscopemap
'''

RETURN = '''
scope_map:
    description:
        - A list of dictionaries containing facts for scope map.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.ContainerRegistry/registries/acr01/scopeMaps/map01"
        name:
            description:
                - The name of the resource.
            returned: always
            type: str
            sample: map01
        registry_name:
            description:
                - The name of the container registry.
            type: str
            returned: always
            sample: registry01
        resource_group:
            description:
                - The name fo the resource group.
            type: str
            returned: always
            sample: myRG
        actions:
            description:
                - The list of scoped permissions for registry artifacts.
            type: list
            returned: always
            sample: ["repositories/test01/content/read"]
        creation_date:
            description:
                - The creation date of scope map.
            type: str
            returned: always
            sample: "2025-07-21T03:15:59.951533Z"
        description:
            description:
                - The user friendly description of the scope map.
            type: str
            returned: always
            sample: "Just for test"
        provisioning_state:
            description:
                - Provisioning state of the resource.
            type: str
            returned: always
            sample: Success
        system_data:
            description:
                - Metadata pertaining to creation and last modification of the resource.
            type: str
            returned: always
            sample: {
                    "created_at": "2025-07-21T03:15:59.287282Z",
                    "created_by": "v-xisu@microsoft.com",
                    "created_by_type": "User",
                    "last_modified_at": "2025-07-21T03:15:59.287282Z",
                    "last_modified_by": "v-xisu@microsoft.com",
                    "last_modified_by_type": "User"
                }
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: 'Microsoft.ContainerRegistry/registries/scopeMaps'
        type_properties_type:
            description:
                - The type of the scope map.
            type: str
            returned: always
            sample: UserDefined
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMContainerRegistryScopeMap(AzureRMModuleBaseExt):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str'
            ),
            actions=dict(
                type='list',
                elements='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False,
            diff=None,
            scope_map=None,
        )
        self.resource_group = None
        self.registry_name = None
        self.name = None
        self.description = None
        self.state = None
        self.actions = None

        super(AzureRMContainerRegistryScopeMap, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        # Defaults for variables
        result = None
        result_compare = dict(compare=[])

        # Get current container registry token
        before_dict = self.get()

        # Create dict form input, without None value
        scope_map_template = dict(description=self.description, actions=self.actions)

        # Filter out all None values
        scope_map_input = {key: value for key, value in scope_map_template.items() if value is not None}

        # Create/Update if state==present
        if self.state == 'present':
            if before_dict:
                # The container registry already exists, try to update
                # Dict for update is the union of existing object over written by input data
                scope_map_update = before_dict | scope_map_input
                if not self.default_compare({}, scope_map_update, before_dict, '', result_compare):
                    self.results['changed'] = True
                    if self.check_mode:
                        # Check mode, skipping actual creation
                        pass
                    else:
                        result = self.update(scope_map_update)
            else:
                self.results['changed'] = True
                # The container registry scope map not exist, create
                if self.check_mode:
                    # Check mode, Skipping actual creation
                    pass
                else:
                    result = self.create(scope_map_input)
        elif self.state == 'absent' and before_dict:
            self.results['changed'] = True
            if not self.check_mode:
                self.delete()
            else:
                # Do not delete in check mode
                pass

        self.results['diff'] = result_compare
        self.results['scope_map'] = result
        return self.results

    def get(self):
        try:
            response = self.containerregistrytoken_client.scope_maps.get(resource_group_name=self.resource_group,
                                                                         registry_name=self.registry_name,
                                                                         scope_map_name=self.name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log("Could not get facts for Registry Scope Map: {0}".format(str(e)))
            return None

        return self.format_item(response)

    def create(self, body):
        try:
            response = self.containerregistrytoken_client.scope_maps.begin_create(resource_group_name=self.resource_group,
                                                                                  registry_name=self.registry_name,
                                                                                  scope_map_name=self.name,
                                                                                  scope_map_create_parameters=body)
            self.log("Response: {0}".format(response))
        except Exception as e:
            self.fail("Create {0} fail. Abnormal message as {1}".format(self.name, str(e)))

        if isinstance(response, LROPoller):
            response = self.get_poller_result(response)

        return self.format_item(response)

    def update(self, body):
        try:
            response = self.containerregistrytoken_client.scope_maps.begin_update(resource_group_name=self.resource_group,
                                                                                  registry_name=self.registry_name,
                                                                                  scope_map_name=self.name,
                                                                                  scope_map_update_parameters=body)
            self.log("Response: {0}".format(response))
        except Exception as e:
            self.fail("Update {0} fail. Abnormal message as {1}".format(self.name, str(e)))

        if isinstance(response, LROPoller):
            response = self.get_poller_result(response)

        return self.format_item(response)

    def delete(self):
        try:
            response = self.containerregistrytoken_client.scope_maps.begin_delete(resource_group_name=self.resource_group,
                                                                                  registry_name=self.registry_name,
                                                                                  scope_map_name=self.name)
            self.log("Response: {0}".format(response))
        except Exception as e:
            self.fail("Deletion {0} fail. Abnormal message as {1}".format(self.name, str(e)))

        if isinstance(response, LROPoller):
            self.get_poller_result(response)

    def format_item(self, item):
        result = item.as_dict()
        result['resource_group'] = self.resource_group
        result['registry_name'] = self.registry_name

        return result


def main():
    AzureRMContainerRegistryScopeMap()


if __name__ == '__main__':
    main()
